<?php

namespace WPSP_PRO;

use WPSP\Helper;
use WPSP_PRO\Helper as WPSP_PROHelper;

class Admin
{
    private $free_enabled = false;
    public function __construct()
    {
        $this->load_plugin_pages();
        $this->admin_notice_loaded();
        $this->load_settings();
        add_filter('plugin_action_links_' . WPSP_PRO_PLUGIN_BASENAME, array($this, 'insert_pro_plugin_links'));
        add_action( 'wpsp_el_action_before', [ $this, 'wpsp_el_action_before' ] );
        add_action( 'wpsp_el_modal_pro_fields', [ $this, 'wpsp_el_modal_pro_fields' ] );
    }

    public function load_plugin_pages()
    {
        new Admin\Menu();
    }

    public function load_settings()
    {
        new Admin\Settings(WPSP_SETTINGS_SLUG, WPSP_SETTINGS_NAME);
    }

    /**
     * Extending plugin links
     *
     * @since 2.3.1
     */
    public function insert_pro_plugin_links($links)
    {
        // settings
        $links[] = sprintf('<a href="admin.php?page=' . WPSP_PRO_SETTINGS_SLUG . '">' . __('Settings', 'wp-scheduled-posts-pro') . '</a>');

        // go pro
        if (!is_plugin_active('wp-scheduled-posts-pro/wp-scheduled-posts-pro.php')) {
            $links[] = sprintf('<a href="https://wpdeveloper.com/plugins/wp-scheduled-posts/" target="_blank" style="color: #39b54a; font-weight: bold;">' . __('Go Pro', 'wp-scheduled-posts-pro') . '</a>');
        }

        return $links;
    }

    public function admin_notice_loaded()
    {
        $this->free_enabled();
        // version compatibility notice
        if ($this->free_enabled && \version_compare(WPSP_PRO_VERSION, '2.5.3', '<')) {
            add_action('admin_notices', array($this, 'admin_version_compatible_notice'));
        }
    }

    /**
     * Check free is enabled
     */

    public function free_enabled()
    {
        if (function_exists('is_plugin_active')) {
            return $this->free_enabled = is_plugin_active('wp-scheduled-posts/wp-scheduled-posts.php');
        } else {
            if (class_exists('WPSP')) {
                return $this->free_enabled = true;
            }
        }
    }

    public function admin_version_compatible_notice()
    {
        $class = 'notice notice-error';
        $message = __('You are using an incompatible version of', 'wp-scheduled-posts-pro') . '<strong> ' . __('SchedulePress Pro.', 'wp-scheduled-posts-pro') . ' </strong>' . __('Please update to v2.5.3+', 'wp-scheduled-posts-pro');
        printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), $message);
    }

	public function wpsp_el_action_before( $args ) {
        if ( ! empty( $args['republish_datetime'] ) ) {
            update_post_meta( $args['id'], '_wpscp_schedule_republish_date', sanitize_text_field( $args['republish_datetime'] ) );
        }

        if ( ! empty( $args['unpublish_datetime'] ) ) {
            update_post_meta( $args['id'], '_wpscp_schedule_draft_date', sanitize_text_field( $args['unpublish_datetime'] ) );
        }
        
        if ( !empty( $args['_wpscppro_advance_schedule_date'] ) ) {
            $scheduled_data = get_post_meta( $args['id'], 'wpscp_pending_schedule', true );
            if( !empty( $scheduled_data['meta']['_wpscppro_advance_schedule_date'] ) ) {
                $scheduled_data['meta']['_wpscppro_advance_schedule_date'] = sanitize_text_field( $args['_wpscppro_advance_schedule_date'] );
                update_post_meta( $args['id'], 'wpscp_pending_schedule', $scheduled_data );
            }
            update_post_meta( $args['id'], '_wpscppro_advance_schedule', true );
            update_post_meta( $args['id'], '_wpscppro_advance_schedule_date', sanitize_text_field( $args['_wpscppro_advance_schedule_date'] ) );
        }else{
            update_post_meta( $args['id'], '_wpscppro_advance_schedule', false );
            update_post_meta( $args['id'], '_wpscppro_advance_schedule_date', null );
        }
	}

    public function wpsp_get_advance_schedule_date( $post_id ) {
        $wpsp_scheduled_data = get_post_meta( $post_id, 'wpscp_pending_schedule',true );
        if( !empty( $wpsp_scheduled_data['meta']['_wpscppro_advance_schedule_date'] ) ) {
            return $wpsp_scheduled_data['meta']['_wpscppro_advance_schedule_date'];
        }
        return get_post_meta( $post_id, '_wpscppro_advance_schedule_date', true);
    }

	public function wpsp_el_modal_pro_fields( $post_id ) {
        $republish_date        = get_post_meta( $post_id, '_wpscp_schedule_republish_date', true );
        $unpublish_date        = get_post_meta( $post_id, '_wpscp_schedule_draft_date', true );
        $advance_schedule_date = null;
        if( WPSP_PROHelper::wpsp_check_advance_schedule( $post_id ) ) {
            $advance_schedule_date = $this->wpsp_get_advance_schedule_date( $post_id );
        }
        // $is_advanced           = get_post_meta($post_id, 'wpscp_el_pending_schedule', true);
        // if( empty( $advance_schedule_date ) && !empty( $is_advanced['post_time'] ) ) {
        //     $advance_schedule_date   = !empty($is_advanced['post_time']) ? $is_advanced['post_time'] : null;
        // }
        $status = get_post_status($post_id);
        $advance_schedule                = get_post_meta( $post_id, '_wpscppro_advance_schedule', true );
        $is_delayed_schedule_active      = ( \WPSP\Helper::get_settings('is_delayed_schedule_active') && $status == 'publish' ) ? true : false;
        $status_post_republish_unpublish = Helper::get_settings('post_republish_unpublish');
        $advanced_schedule_url = admin_url() . 'admin.php?page=schedulepress&tab="advanced-schedule"';
        $general_tab_url = admin_url() . 'admin.php?page=schedulepress&tab="general"';

        ?>
        <div class="wpsp-pro-fields wpsp-pro-tooltip wpsp-pro-activated <?php echo ($status_post_republish_unpublish !== true) ? 'disabled' : '';  ?>">
            <label>
                <span><?php esc_html_e( 'Unpublish On', 'wp-scheduled-posts' ); ?></span>
                <a href="<?php echo esc_url('https://wpdeveloper.com/docs/republish-or-unpublish-post-from-elementor/') ?>" class="advance-schedule-info" target="_blank"><span class="dashicons dashicons-info"></span></a>
                <div class="wpsp-date-input-wrapper">
                    <input id="wpsp-unpublish-datetime" placeholder="<?php echo esc_attr('Y/M/D H:M:S') ?>" type="text" name="unpublish_datetime" value="<?php echo esc_attr( $unpublish_date ) ?>" <?php echo ($status_post_republish_unpublish !== true) ? 'disabled' : '';  ?> readonly autocomplete="off">
                    <?php if( empty( $status_post_republish_unpublish ) ) : ?>
                        <span class="wpsp-el-disabled-text tooltiptext"><?php echo sprintf( __('Unpublish option is disabled, to enable it go <a href="%s" target="_blank">SchedulePress Settings</a> ', 'wp-scheduled-posts-pro'), esc_url( $general_tab_url ) ) ?></span>
                    <?php endif ?>
                </div>
            </label>
            <label>
                <span><?php esc_html_e( 'Republish On', 'wp-scheduled-posts' ); ?></span>
                <a href="<?php echo esc_url('https://wpdeveloper.com/docs/republish-or-unpublish-post-from-elementor/') ?>" class="advance-schedule-info" target="_blank"><span class="dashicons dashicons-info"></span></a>
                <div class="wpsp-date-input-wrapper">
                    <input id="wpsp-republish-datetime" placeholder="<?php echo esc_attr('Y/M/D H:M:S') ?>" type="text" name="republish_datetime" value="<?php echo esc_attr( $republish_date ) ?>" <?php echo ($status_post_republish_unpublish !== true) ? 'disabled' : '';  ?>  readonly autocomplete="off">
                    <?php if( empty( $status_post_republish_unpublish ) ) : ?>
                        <span class="wpsp-el-disabled-text tooltiptext"><?php echo sprintf( __('Republish option is disabled, to enable it go <a href="%s" target="_blank">SchedulePress Settings</a> ', 'wp-scheduled-posts-pro'), esc_url( $general_tab_url ) ) ?></span>
                    <?php endif ?>
                </div>
            </label>
        </div>
        <div class="wpsp-pro-fields wpsp-pro-tooltip wpsp-pro-activated <?php echo empty($is_delayed_schedule_active) ? 'disabled' : '';  ?>">
            <label>
                <span>
                    <?php esc_html_e( 'Advanced Schedule', 'wp-scheduled-posts' ); ?>
                </span>
                <a href="<?php echo esc_url('https://wpdeveloper.com/docs/advanced-schedule-update-published-posts/') ?>" class="advance-schedule-info" target="_blank"><span class="dashicons dashicons-info"></span></a>
                <div class="wpsp-date-input-wrapper">
                    <input id="wpsp-advanced-schedule-datetime" placeholder="<?php echo esc_attr('Y/M/D H:M:S') ?>" type="text" name="_wpscppro_advance_schedule_date" value="<?php echo esc_attr( $advance_schedule_date ) ?>" <?php echo empty($is_delayed_schedule_active) ? 'disabled' : '';  ?> readonly autocomplete="off">
                    <?php if( empty( $is_delayed_schedule_active ) ) : ?>
                        <span class="wpsp-el-disabled-text tooltiptext"><?php echo sprintf( __('Advanced Schedule is disabled, to enable it go <a href="%s" target="_blank">SchedulePress Settings</a> ', 'wp-scheduled-posts-pro'), esc_url( $advanced_schedule_url ) ) ?></span>
                    <?php endif ?>
                </div>
            </label>
        </div>
		<?php
	}
}
